#!/bin/bash

print_env.sh

function print_the_help {
  echo "USAGE: ${0} -n <nevents> -t <nametag> -p <particle> "
  echo "  OPTIONS: "
  echo "    -n,--nevents     Number of events"
  echo "    -t,--nametag     name tag"
  echo "    -p,--particle    particle type"
  echo "                     allowed types: pion0, pion+, pion-, kaon0, kaon+, kaon-, proton, neutron, electron, positron, photon"
  echo "    --pmin           minimum particle momentum (GeV)"
  echo "    --pmax           maximum particle momentum (GeV)"
  echo "    --angmin         minimum particle angle (deg)"
  echo "    --angmax         maximum particle angle (deg)"
  echo "    --region         detector region"
  echo "                     allowed regions: farbackward, backward, barrel, forward, farforward"
  exit
}

POSITIONAL=()
while [[ $# -gt 0 ]]
do
  key="$1"

  case $key in
    -h|--help)
      shift # past argument
      print_the_help
      ;;
    -t|--nametag)
      nametag="$2"
      shift # past argument
      shift # past value
      ;;
    -p|--particle)
      particle="$2"
      shift # past argument
      shift # past value
      ;;
    -n|--nevents)
      export JUGGLER_N_EVENTS="$2"
      shift # past argument
      shift # past value
      ;;
    --pmin)
      export PMIN="$2"
      shift
      shift
      ;;
    --pmax)
      export PMAX="$2"
      shift
      shift
      ;;
    --angmin)
      export ANGMIN="$2"
      shift
      shift
      ;;
    --angmax)
      export ANGMAX="$2"
      shift
      shift
      ;;
    --region)
      export REGION="$2"
      shift
      shift
      ;;
    *)    # unknown option
      #POSITIONAL+=("$1") # save it in an array for later
      echo "unknown option $1"
      print_the_help
      shift # past argument
      ;;
  esac
done
set -- "${POSITIONAL[@]}" # restore positional parameters

if [[ ! -n  "${JUGGLER_N_EVENTS}" ]] ; then
  export JUGGLER_N_EVENTS=1000
fi

if [[ ! -n  "${PARTICLE}" ]] ; then
  export PARTICLE="electron"
fi

if [[ ! -n  "${PMIN}" ]] ; then
  export PMIN=1.0
fi

if [[ ! -n  "${PMAX}" ]] ; then
  export PMAX=10.0
fi

if [[ ! -n  "${ANGMIN}" ]] ; then
  export ANGMIN=3
fi

if [[ ! -n  "${ANGMAX}" ]] ; then
  export ANGMAX=177
fi

if [[ ! -n  "${REGION}" ]] ; then
  export REGION="ignore"
fi

if [[ ! -n "${JUGGLER_DETECTOR}" ]] ; then
  export JUGGLER_DETECTOR="athena"
fi

if [[ ! -n "${JUGGLER_INSTALL_PREFIX}" ]] ; then
  export JUGGLER_INSTALL_PREFIX="/usr/local"
fi

compact_path=${DETECTOR_PATH}/${JUGGLER_DETECTOR}.xml
export JUGGLER_FILE_NAME_TAG="${nametag}"
export JUGGLER_GEN_FILE="gen_${JUGGLER_FILE_NAME_TAG}.hepmc"

export JUGGLER_SIM_FILE="sim_${JUGGLER_FILE_NAME_TAG}.root"
export JUGGLER_REC_FILE="rec_${JUGGLER_FILE_NAME_TAG}.root"

echo "JUGGLER_N_EVENTS = ${JUGGLER_N_EVENTS}"
echo "DETECTOR_PATH = ${DETECTOR_PATH}"
echo "JUGGLER_DETECTOR = ${JUGGLER_DETECTOR}"

# Generate the input events
if [ ! -f ${JUGGLER_GEN_FILE} ] ; then
  python benchmarks/full/scripts/gen_particles.py ${JUGGLER_GEN_FILE} -n ${JUGGLER_N_EVENTS}\
    --angmin ${ANGMIN} --angmax ${ANGMAX} --phmin 0 --phmax 360 --pmin ${PMIN} --pmax ${PMAX}\
    --regions="${REGION}" --particles="${particle}"
fi

if [[ "$?" -ne "0" ]] ; then
  echo "ERROR running script: generating input events"
  exit 1
fi

ls -lh ${JUGGLER_GEN_FILE}

# Run geant4 simulations
if [ ! -f ${JUGGLER_SIM_FILE} ] ; then
  npsim --runType batch \
        -v WARNING \
        --part.minimalKineticEnergy "1*TeV" \
        --numberOfEvents ${JUGGLER_N_EVENTS} \
        --compactFile ${compact_path} \
        --inputFiles ${JUGGLER_GEN_FILE} \
        --outputFile ${JUGGLER_SIM_FILE}
fi

if [[ "$?" -ne "0" ]] ; then
  echo "ERROR running npdet"
  exit 1
fi

rootls -t "${JUGGLER_SIM_FILE}"

# Directory for results
mkdir -p results

OPTION_DIR=benchmarks/full/options
SCRIPT_DIR=benchmarks/full/scripts

# Run Juggler
gaudirun.py ${OPTION_DIR}/full_reconstruction.py
if [[ "$?" -ne "0" ]] ; then
  echo "ERROR running reconstruction (juggler)"
  exit 1
fi

rootls -t "${JUGGLER_REC_FILE}"

# Run analysis scripts
#python ${SCRIPT_DIR}/full_reconstruction.py ${JUGGLER_SIM_FILE} ${JUGGLER_REC_FILE} -o results

root_filesize=$(stat --format=%s "${JUGGLER_REC_FILE}")
if [[ "${JUGGLER_N_EVENTS}" -lt "500" ]] ; then
  # file must be less than 10 MB to upload
  if [[ "${root_filesize}" -lt "10000000" ]] ; then
    cp ${JUGGLER_REC_FILE} results/.
  fi
fi

