import json
import os
import ROOT
from Gaudi.Configuration import *
from GaudiKernel.SystemOfUnits import MeV, mm, cm, mrad, rad, ns
from Configurables import ApplicationMgr, EICDataSvc, PodioOutput, GeoSvc

from Configurables import PodioInput
from Configurables import Jug__Base__InputCopier_dd4pod__Geant4ParticleCollection_dd4pod__Geant4ParticleCollection_ as MCCopier
from Configurables import Jug__Digi__CalorimeterHitDigi as CalHitDigi
from Configurables import Jug__Reco__CalorimeterHitReco as CalHitReco
from Configurables import Jug__Reco__CalorimeterHitsMerger as CalHitsMerger
from Configurables import Jug__Reco__CalorimeterIslandCluster as IslandCluster
from Configurables import Jug__Reco__ClusterRecoCoG as RecoCoG
from Configurables import Jug__Reco__ImagingPixelReco as ImagingPixelReco
from Configurables import Jug__Reco__ImagingTopoCluster as ImagingTopoCluster
from Configurables import Jug__Reco__ImagingClusterReco as ImagingClusterReco

# input arguments from calibration file
with open('config/emcal_barrel_calibration.json') as f:
    calib_data = json.load(f)['electron']

print(calib_data)

kwargs = dict()
kwargs['img_sf'] = float(calib_data['sampling_fraction_img'])
kwargs['scfi_sf'] = float(calib_data['sampling_fraction_scfi'])

# input arguments through environment variables
kwargs['input'] = os.environ['CB_EMCAL_SIM_FILE']
kwargs['output'] = os.environ['CB_EMCAL_REC_FILE']
kwargs['compact'] = os.environ['CB_EMCAL_COMPACT_PATH']
kwargs['nev'] = int(os.environ['CB_EMCAL_NUMEV'])

if kwargs['nev'] < 1:
    f = ROOT.TFile(kwargs['input'])
    kwargs['nev'] = f.events.GetEntries()

print(kwargs)

# get sampling fraction from system environment variable, 1.0 by default
geo_service = GeoSvc('GeoSvc', detectors=kwargs['compact'].split(','), OutputLevel=INFO)
podioevent = EICDataSvc('EventDataSvc', inputs=kwargs['input'].split(','), OutputLevel=DEBUG)

sim_colls = [
    'mcparticles',
    'EcalBarrelHits',
    'EcalBarrelScFiHits',
]
podin = PodioInput('PodioReader', collections=sim_colls, OutputLevel=DEBUG)
podout = PodioOutput('podout', filename=kwargs['output'])

mccopier = MCCopier('MCCopier',
        # OutputLevel=DEBUG,
        inputCollection='mcparticles',
        outputCollection='mcparticles2')

# use the same daq_setting for digi/reco pair
imcaldaq = dict(
        dynamicRangeADC=3*MeV,
        capacityADC=32767,
        pedestalMean=400,
        pedestalSigma=50)   # 50/32767*3 MeV ~ 5 keV

imcaldigi = CalHitDigi('imcal_digi',
        # OutputLevel=DEBUG,
        inputHitCollection='EcalBarrelHits',
        outputHitCollection='DigiEcalBarrelImagingHits',
        energyResolutions=[0., 0.02, 0.],
        **imcaldaq)
imcalreco = ImagingPixelReco('imcal_reco',
        # OutputLevel=DEBUG,
        inputHitCollection=imcaldigi.outputHitCollection,
        outputHitCollection='RecoEcalBarrelImagingHits',
        readoutClass='EcalBarrelHits',
        layerField='layer',
        sectorField='module',
        **imcaldaq)

imcalcluster = ImagingTopoCluster('imcal_cluster',
        # OutputLevel=DEBUG,
        inputHitCollection=imcalreco.outputHitCollection,
        outputProtoClusterCollection='EcalBarrelImagingProtoClusters',
        localDistXY=[2.*mm, 2*mm],
        layerDistEtaPhi=[10*mrad, 10*mrad],
        neighbourLayersRange=2,
        sectorDist=3.*cm)
clusterreco = ImagingClusterReco('imcal_clreco',
        # OutputLevel=DEBUG,
        inputHitCollection=imcalcluster.inputHitCollection,
        inputProtoClusterCollection=imcalcluster.outputProtoClusterCollection,
        outputLayerCollection='EcalBarrelImagingClustersLayers',
        outputClusterCollection='EcalBarrelImagingClusters',
        outputInfoCollection='EcalBarrelImagingClustersInfo',
        samplingFraction=kwargs['img_sf'])

# scfi layers
scfi_barrel_daq = dict(
        dynamicRangeADC=50.*MeV,
        capacityADC=32768,
        pedestalMean=400,
        pedestalSigma=10)

scfi_barrel_digi = CalHitDigi("scfi_barrel_digi",
        inputHitCollection="EcalBarrelScFiHits",
        outputHitCollection="EcalBarrelScFiHitsDigi",
        **scfi_barrel_daq)

scfi_barrel_reco = CalHitReco("scfi_barrel_reco",
        inputHitCollection=scfi_barrel_digi.outputHitCollection,
        outputHitCollection="EcalBarrelScFiHitsReco",
        thresholdFactor=5.0,
        readoutClass="EcalBarrelScFiHits",
        layerField="layer",
        sectorField="module",
        localDetFields=["system", "module"], # use local coordinates in each module (stave)
        **scfi_barrel_daq)

# merge hits in different layer (projection to local x-y plane)
scfi_barrel_merger = CalHitsMerger("scfi_barrel_merger",
        # OutputLevel=DEBUG,
        inputHitCollection=scfi_barrel_reco.outputHitCollection,
        outputHitCollection="EcalBarrelScFiGridReco",
        fields=["fiber"],
        fieldRefNumbers=[1],
        readoutClass="EcalBarrelScFiHits")

scfi_barrel_cl = IslandCluster("scfi_barrel_cl",
        # OutputLevel=DEBUG,
        inputHitCollection=scfi_barrel_merger.outputHitCollection,
        outputProtoClusterCollection="EcalBarrelScFiProtoClusters",
        splitCluster=False,
        minClusterCenterEdep=10.*MeV,
        localDistXZ=[30*mm, 30*mm],
        sectorDist=5.*cm)

scfi_barrel_clreco = RecoCoG("scfi_barrel_clreco",
       inputHitCollection=scfi_barrel_cl.inputHitCollection,
       inputProtoClusterCollection=scfi_barrel_cl.outputProtoClusterCollection,
       outputClusterCollection="EcalBarrelScFiClusters",
       outputInfoCollection="EcalBarrelScFiClustersInfo",
       logWeightBase=6.2,
       samplingFraction=kwargs['scfi_sf'])

# TODO: merge two types of clusters

podout.outputCommands = ['drop *',
        'keep mcparticles2',
        'keep *Reco*',
        'keep *Digi*',
        'keep *Cluster*',
        'keep *Layer*',
        ]

ApplicationMgr(
    TopAlg=[podin, mccopier,
        imcaldigi, imcalreco, imcalcluster, clusterreco,
        scfi_barrel_digi, scfi_barrel_reco, scfi_barrel_merger, scfi_barrel_cl, scfi_barrel_clreco,
        podout],
    EvtSel='NONE',
    EvtMax=kwargs['nev'],
    ExtSvc=[podioevent],
    OutputLevel=DEBUG
)
